@@ -1,6 +1,6 @@
 /*
  * preferences.cc - preferences related code for Bombermaze
- * written by Sydney Tang <sydney.tang@computer.org>
+ * written by Sydney Tang <stang@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,11 +48,11 @@
 
 //////////////////////////////////////////////////////////////////////////////
 
-const int MAP_PREVIEW_WIDTH = 320;
+const int MAP_PREVIEW_WIDTH = 380;
 const int MAP_PREVIEW_HEIGHT = 320;
 
-const int THEME_PREVIEW_WIDTH = 320;
-const int THEME_PREVIEW_HEIGHT = 320;
+const int THEME_PREVIEW_WIDTH = 400;
+const int THEME_PREVIEW_HEIGHT = 150;
 
 const gchar *GENERAL_CONFIGPATH_SECTION = "/"PACKAGE"/General/";
 
@@ -323,13 +323,17 @@ static KeyMap TempKeyPreferences;
 static int TempPowerUpProbability[PowerUp::NUMBER_OF_CREATABLE_POWERUPS];
 
 static GameMap PreviewMap;
+static Player MapPreviewPlayers[Player::MAX_NUMBER_OF_PLAYERS];
 static GtkWidget *MapPreviewArea = NULL;
 static GdkPixmap *MapPreviewPixmap = NULL;
 static UIDisplayInterface MapPreviewDisplay = { NULL, NULL };
 
+static GtkWidget *ThemePreviewPlaceHolder = NULL;
 static GtkWidget *ThemePreviewArea = NULL;
 static GdkPixmap *ThemePreviewPixmap = NULL;
 static UIDisplayInterface ThemePreviewDisplay = { NULL, NULL };
+static GtkWidget *ThemePreviewText = NULL;
+
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -378,6 +382,7 @@ static void map_selection (GtkWidget *item, gpointer user_data);
 static void map_preview_update(const char *map_path);
 static void theme_selection (GtkWidget *item, gpointer user_data);
 static void theme_preview_update(const char *theme_path);
+static void map_preview_update_initialize_players(GameMap *maze, Player *p);
 static void check_button_click(GtkWidget *widget, gpointer data);
 static void spinner_keypress( GtkWidget *widget,
                               GdkEventKey *event,
@@ -1398,6 +1403,8 @@ void map_preview_update(const char *map_path)
     int MazePixelWidth =PreviewMap.get_width()*MapSquare::get_square_width();
     int MazePixelHeight=PreviewMap.get_height()*MapSquare::get_square_height();
 
+    map_preview_update_initialize_players(&PreviewMap, MapPreviewPlayers);
+
     // set up a new target pixmap for the GameMap to draw to
     GdkPixmap *map_pixmap = gdk_pixmap_new (MapPreviewDisplay.window,
                                             MazePixelWidth,
@@ -1465,6 +1472,67 @@ void theme_selection (GtkWidget *item, gpointer user_data)
 
 void theme_preview_update(const char *theme_path)
 {
+  gtk_editable_delete_text (GTK_EDITABLE(ThemePreviewText), 0, -1);
+  int theme_description_length;
+  char *theme_description;
+
+  gchar *theme_description_file = g_strdup_printf("%s/README", theme_path);
+
+  FILE *fp = fopen(theme_description_file, "r");
+  g_free(theme_description_file);
+  if (fp == NULL)
+  {
+    return;
+  }
+
+  int buffer_size = 1024;
+  for (;;)
+  {
+    theme_description = new char[buffer_size];
+    theme_description_length = fread(theme_description, sizeof(char),
+                                     buffer_size-1, fp);
+    if (theme_description_length >= buffer_size - 1)
+    {
+      fseek(fp, 0, SEEK_SET);
+      delete theme_description;
+      buffer_size *= 2;
+    }
+    else
+    {
+      theme_description[theme_description_length] = '\0';
+      break;
+    }
+  }
+
+  int theme_description_position = 0;
+
+  gtk_editable_insert_text (GTK_EDITABLE(ThemePreviewText),
+                            theme_description,
+                            theme_description_length,
+                            &theme_description_position);
+  gtk_editable_set_position (GTK_EDITABLE(ThemePreviewText), 0);
+
+  fclose(fp);
+  delete theme_description;
+}
+
+void map_preview_update_initialize_players(GameMap *maze, Player *player)
+{
+  int i;
+  int start_x, start_y;
+  int n_players = maze->get_allowable_number_of_players();
+
+  for (i = 0; i < n_players; i++)
+  {
+    player[i].reset_fields();
+    player[i].set_id(i);
+    player[i].set_game_map(maze);
+    maze->get_starting_coordinates_of_player(i+1, start_x, start_y);
+    player[i].set_starting_coordinates(start_x, start_y);
+    player[i].activate();
+    maze->move_entity_to_top(&(player[i]), start_x, start_y);
+  }
+  return;
 }
 
 void check_button_click(GtkWidget *widget, gpointer data)
@@ -1927,7 +1995,7 @@ void append_page_theme(GtkWidget *PropertyBox)
 {
   GtkWidget *label = gtk_label_new(_("Theme"));
   gtk_widget_show(label);
-  GtkWidget *table = gtk_table_new (2, 2, FALSE);
+  GtkWidget *table = gtk_table_new (3, 2, FALSE);
   preferences_set_standard_table_layout(table);
   gnome_property_box_append_page (GNOME_PROPERTY_BOX(PropertyBox),
                                   table,
@@ -1957,26 +2025,77 @@ void append_page_theme(GtkWidget *PropertyBox)
   gtk_table_attach (GTK_TABLE(table), button, 0, 1, 1, 2,
                     GTK_SHRINK, GTK_SHRINK, 0, 0);
 
+  GtkWidget *table2;
+  
   frame = gtk_frame_new(_("Preview"));
   gtk_container_border_width (GTK_CONTAINER (frame), GNOME_PAD);
   gtk_widget_show(frame);
+
   gtk_table_attach (GTK_TABLE(table), frame, 1, 2, 0, 2,
                     GTK_FILL, GTK_FILL, 0, 0);
 
+  table2 = gtk_table_new (2, 1, FALSE);
+  preferences_set_standard_table_layout(table2);
+  gtk_container_add (GTK_CONTAINER(frame), table2);
+
   ThemePreviewArea = gtk_drawing_area_new();
   gtk_drawing_area_size ((GtkDrawingArea *)ThemePreviewArea,
-                         MAP_PREVIEW_WIDTH,
-                         MAP_PREVIEW_HEIGHT);
+                         THEME_PREVIEW_WIDTH,
+                         THEME_PREVIEW_HEIGHT);
   gtk_widget_set_usize ((GtkWidget *)ThemePreviewArea,
-                         MAP_PREVIEW_WIDTH,
-                         MAP_PREVIEW_HEIGHT);
+                         THEME_PREVIEW_WIDTH,
+                         THEME_PREVIEW_HEIGHT);
   gtk_signal_connect (GTK_OBJECT (ThemePreviewArea), "realize",
                       GTK_SIGNAL_FUNC (realize), &ThemePreviewDisplay);
   gtk_signal_connect (GTK_OBJECT (ThemePreviewArea), "expose_event",
                       GTK_SIGNAL_FUNC (expose_event), &ThemePreviewPixmap);
   gtk_widget_show (ThemePreviewArea);
-  gtk_container_add (GTK_CONTAINER(frame), ThemePreviewArea);
+  gtk_table_attach (GTK_TABLE(table2), ThemePreviewArea, 0, 1, 0, 1,
+                    GTK_FILL, GTK_FILL, 0, 0);
+  //gtk_container_add (GTK_CONTAINER(frame), ThemePreviewArea);
   gtk_widget_realize (ThemePreviewArea);
+
+  ThemePreviewPlaceHolder = gtk_label_new("not implemented yet");
+  gtk_widget_show (ThemePreviewPlaceHolder);
+  //gtk_container_add (GTK_CONTAINER(frame), ThemePreviewPlaceHolder);
+  gtk_table_attach (GTK_TABLE(table2), ThemePreviewPlaceHolder, 0, 1, 1, 2,
+                    GTK_FILL, GTK_FILL, 0, 0);
+
+  
+  table2 = gtk_hbox_new(FALSE, 0);
+  //table2 = gtk_table_new (2, 2, FALSE);
+  //preferences_set_standard_table_layout(table2);
+
+  ThemePreviewText = gtk_text_new(NULL, NULL);
+  gtk_text_set_editable (GTK_TEXT(ThemePreviewText), FALSE);
+  gtk_text_set_word_wrap (GTK_TEXT(ThemePreviewText), TRUE);
+
+  gtk_widget_show(ThemePreviewText);
+  gtk_box_pack_start(GTK_BOX(table2), ThemePreviewText, TRUE, TRUE, 0);
+  //gtk_table_attach (GTK_TABLE(table2), ThemePreviewText, 0, 1, 0, 1,
+  //                  GTK_FILL, GTK_FILL, 0, 0);
+
+  GtkWidget *scrollbar;
+
+  scrollbar =
+    gtk_vscrollbar_new (GTK_ADJUSTMENT(GTK_TEXT(ThemePreviewText)->vadj));
+  gtk_widget_show(scrollbar);
+  gtk_box_pack_end(GTK_BOX(table2), scrollbar, FALSE, TRUE, 0);
+  //gtk_table_attach (GTK_TABLE(table2), scrollbar, 1, 2, 0, 1,
+  //                  GTK_SHRINK, GTK_FILL, 0, 0);
+
+  /*
+  scrollbar =
+    gtk_hscrollbar_new (GTK_ADJUSTMENT(GTK_TEXT(ThemePreviewText)->hadj));
+  gtk_widget_show(scrollbar);
+  gtk_table_attach (GTK_TABLE(table2), scrollbar, 0, 1, 1, 2,
+                    GTK_FILL, GTK_SHRINK, 0, 0);
+  */
+
+  gtk_widget_show(table2);
+  gtk_table_attach (GTK_TABLE(table), table2, 0, 2, 2, 3,
+                    GTK_FILL, GTK_FILL, 0, 0);
+
   theme_preview_update(preferences.Theme);
 }
 
