@@ -43,7 +43,7 @@ const char  *WINDOW_TITLE = N_("BomberMaze");
 const gchar *COPYRIGHT = N_("(C) 2000 Sydney Tang");
 const gchar *AUTHORS[] = { "Sydney Tang", NULL };
 const gchar *COMMENTS  = N_("Send comments and bug reports to: \n"
-                           " sydney.tang@computer.org\n");
+                           " stang@users.sourceforge.net\n");
 const gchar *LOGO = NULL;
 
 const int   INITIAL_DRAWING_AREA_WIDTH  = 720;
